/*
 * Definitions for use with the sw_64 wrperfmon PAL call.
 */

#ifndef __SW_64_WRPERFMON_H
#define __SW_64_WRPERFMON_H

/* Following commands are implemented on all CPUs */
#define PERFMON_CMD_DISABLE 0
#define PERFMON_CMD_ENABLE 1
#define PERFMON_CMD_DESIRED_EVENTS 2
#define PERFMON_CMD_LOGGING_OPTIONS 3
/* Following commands on SW2/SW2F/PCA56 only */
#define PERFMON_CMD_INT_FREQ 4
#define PERFMON_CMD_ENABLE_CLEAR 7
/* Following commands are on SW2 and better CPUs */
#define PERFMON_CMD_READ 5
#define PERFMON_CMD_WRITE 6
/* Following command are on SW2 and better CPUs */
#define PERFMON_CMD_ENABLE_WRITE 7
/* Following command are on SW2F and better CPUs */
#define PERFMON_CMD_I_STAT 8
#define PERFMON_CMD_PMPC 9


/* SW2/SW2F/PCA56 Counters */
#define SW2_PCTR_0 (1UL<<0)
#define SW2_PCTR_1 (1UL<<1)
#define SW2_PCTR_2 (1UL<<2)

#define SW2_PCTR_0_COUNT_SHIFT 48
#define SW2_PCTR_1_COUNT_SHIFT 32
#define SW2_PCTR_2_COUNT_SHIFT 16

#define SW2_PCTR_0_COUNT_MASK 0xffffUL
#define SW2_PCTR_1_COUNT_MASK 0xffffUL
#define SW2_PCTR_2_COUNT_MASK 0x3fffUL

#if 0
/* SW2 Counters */
#define SW2_PCTR_0 (1UL<<0)
#define SW2_PCTR_1 (1UL<<1)

#define SW2_PCTR_0_COUNT_SHIFT 28
#define SW2_PCTR_1_COUNT_SHIFT 6

#define SW2_PCTR_0_COUNT_MASK 0xfffffUL
#define SW2_PCTR_1_COUNT_MASK 0xfffffUL
#endif

/* SW2F (and subsequent) counters */
#define SW2F_PCTR_0 (1UL<<0)
#define SW2F_PCTR_1 (1UL<<1)

#define SW2F_PCTR_0_COUNT_SHIFT 28
#define SW2F_PCTR_1_COUNT_SHIFT 6

#define SW2F_PCTR_0_COUNT_MASK 0xfffffUL
#define SW2F_PCTR_1_COUNT_MASK 0xfffffUL


/*
 * The sw_64 Architecure Handbook, vers. 4 (1998) appears to have a misprint
 *  in Table E-23 regarding the bits that set the event PCTR 1 counts.
 *  Hopefully what we have here is correct.
 */
#define SW2_PCTR_0_EVENT_MASK 0x10UL
#define SW2_PCTR_1_EVENT_MASK 0x0fUL

/* SW2 Events */
#define SW2_PCTR_0_CYCLES (0UL << 4)
#define SW2_PCTR_0_INSTRUCTIONS (1UL << 4)

#define SW2_PCTR_1_CYCLES 0
#define SW2_PCTR_1_BRANCHES 1
#define SW2_PCTR_1_BRANCH_MISPREDICTS 2
#define SW2_PCTR_1_DTB_SINGLE_MISSES 3
#define SW2_PCTR_1_DTB_DOUBLE_MISSES 4
#define SW2_PCTR_1_ITB_MISSES 5
#define SW2_PCTR_1_UNALIGNED_TRAPS 6
#define SW2_PCTR_1_REPLY_TRAPS 7

/* From the sw_64 Architecture Reference Manual, 4th edn., 2002 */
#define SW2F_PCTR_MODE_MASK 0x10UL
#define SW2F_PCTR_EVENT_MASK 0x0CUL

#define SW2F_PCTR_MODE_PROFILEME (1UL<<4)
#define SW2F_PCTR_MODE_AGGREGATE (0UL<<4)

#define SW2F_PCTR_INSTR_CYCLES (0UL<<2)
#define SW2F_PCTR_CYCLES_UNDEF (1UL<<2)
#define SW2F_PCTR_INSTR_BCACHEMISS (2UL<<2)
#define SW2F_PCTR_CYCLES_MBOX (3UL<<2)

#endif
